Public Class PixelGrid
    ' dies wäre das eigentliche Control, das die Farbflächen zeichnet
    ' Achtung, dieses Control ist nicht Performance-optimiert!

    ' 10000 Farbflächen
    Private m_Elemente(99, 99) As Color
    ' aktive Zeichenfarbe
    Private m_ActiveColor As Color = Color.Red

    ' Aktuelle Zeile und Spalte für die Mausauswahl,
    ' um beim Bewegen der Maus nicht ständig unnötige Aktionen durchzuführen
    Private m_BekannteZeile As Integer = -1
    Private m_BekannteSpalte As Integer = -1

    ''' <summary>
    ''' gibt die aktive Zeichenfarbe zurück oder legt diese fest
    ''' </summary>
    Property ActiveColor() As Color
        Get
            Return m_ActiveColor
        End Get
        Set(ByVal value As Color)
            m_ActiveColor = value
            m_BekannteZeile = -1
            m_BekannteSpalte = -1
        End Set
    End Property

    ''' <summary>
    ''' legt für alle Farbflächen Weiß als Farbe fest
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub ClearAll()
        For I As Integer = 0 To 99
            For J As Integer = 0 To 99
                m_Elemente(I, J) = Color.White
            Next
        Next
        ' Control neu zeichnen
        Invalidate()
    End Sub

    ''' <summary>
    ''' Konstruktor
    ''' </summary>
    ''' <remarks></remarks>
    Public Sub New()
        ' Dieser Aufruf ist für den Windows Form-Designer erforderlich.
        InitializeComponent()
        ' Fügen Sie Initialisierungen nach dem InitializeComponent()-Aufruf hinzu.

        ' DoubleBuffering aktivieren, um Flimmern zu vermeiden
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint, True)
        Me.AutoScrollMinSize = New Size(1599, 1599)

        ' beim Start alle Flächen mit weißer Farbe versehen
        For I As Integer = 0 To 99
            For J As Integer = 0 To 99
                m_Elemente(I, J) = Color.White
            Next
        Next
    End Sub


    ''' <summary>
    ''' Überschreiben der OnPaint-Methode, um den angezeigten Inhalt zu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal pe As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(pe)

        ' Referenz zum Graphics-Objekt wird mit den PaintEventArgs geliefert
        Dim g As Graphics = pe.Graphics

        ' Es macht sich leichter, das Graphics-Objekt um die Scroll-Position
        ' zu verschieben, als jedes gezeichnete Element der Scroll-Position einzeln
        ' anzupassen
        g.TranslateTransform(Me.AutoScrollPosition.X, Me.AutoScrollPosition.Y)

        ' Hintergrund weiß füllen
        g.Clear(Color.White)

        ' schwarze vertikale und horizontale Linien zeichnen
        Dim BlackPen As New Pen(Color.Black)
        For I As Integer = 1 To 99
            Dim xWert As Integer = I * 16 - 1
            Dim yWert As Integer = I * 16 - 1
            g.DrawLine(BlackPen, xWert, 0, xWert, 1599)
            g.DrawLine(BlackPen, 0, yWert, 1599, yWert)
        Next
        BlackPen.Dispose()

        ' farbige Rechtecke zeichnen
        For I As Integer = 0 To 99
            For J As Integer = 0 To 99
                Dim WertI As Integer = I * 16
                Dim WertJ As Integer = J * 16
                Dim Rect As New Rectangle(WertI, WertJ, 14, 14)
                If g.ClipBounds.IntersectsWith(Rect) Then
                    Dim Brsh As New SolidBrush(m_Elemente(I, J))
                    g.FillRectangle(Brsh, WertI, WertJ, 15, 15)
                    Brsh.Dispose()
                End If
            Next
        Next
    End Sub

    ''' <summary>
    ''' Handler für das MouseDown-Event des Controls
    ''' </summary>
    Private Sub PixelGrid_MouseDown(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseDown
        If e.Button <> Windows.Forms.MouseButtons.Left Then Exit Sub
        PunktAendern(e.Location)
    End Sub

    ''' <summary>
    ''' Handler für das MouseMove-Event des Controls
    ''' </summary>
    Private Sub PixelGrid_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseMove
        If e.Button <> Windows.Forms.MouseButtons.Left Then Exit Sub
        PunktAendern(e.Location)
    End Sub


    ''' <summary>
    ''' führt entweder nach MouseDown oder MouseMove wenn notwendig zum
    ''' Neu-Festlegen der Farbe eines Pixels
    ''' </summary>
    Private Sub PunktAendern(ByVal MousePosition As Point)
        Dim Spalte As Integer = (MousePosition.X - Me.AutoScrollPosition.X) \ 16
        Dim Zeile As Integer = (MousePosition.Y - Me.AutoScrollPosition.Y) \ 16
        ' nur wenn hier einen neue Zeile oder Spalte mit der Maus tangiert wird
        If m_BekannteZeile = Zeile AndAlso m_BekannteSpalte = Spalte Then Exit Sub
        ' nur wenn die entsprechende Zelle wirklich eine neue Farbe erhalten soll
        If m_Elemente(Spalte, Zeile) <> m_ActiveColor Then
            ' dieser Zelle die neue Farbe übergeben
            m_Elemente(Spalte, Zeile) = m_ActiveColor
            ' Rechteck für diese Zelle ermitteln
            Dim Rect As New Rectangle((Spalte * 16) + Me.AutoScrollPosition.X, (Zeile * 16) + Me.AutoScrollPosition.Y, 16, 16)
            ' Das Control zum Neu-Zeichnen dieses Rechtecks auffordern
            Me.Invalidate(Rect)
            ' die gerade geänderte Zeile und Spalte merken, um hier nicht noch 
            ' einen Durchlauf zu verursachen
            m_BekannteZeile = Zeile
            m_BekannteSpalte = Spalte
        End If
    End Sub


    Private Sub PixelGrid_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        ' nach Größenänderung Control zum Neu-Zeichnen auffordern
        Invalidate()
    End Sub

    Private Sub PixelGrid_Scroll(ByVal sender As Object, ByVal e As System.Windows.Forms.ScrollEventArgs) Handles Me.Scroll
        ' nach Scrollen Control zum Neu-Zeichnen auffordern
        Invalidate()
    End Sub
End Class
